function RunEFO
%%   Electric Fish Optimization Algorithm, @version 1.0 (07/2017)
%
%   Electric Fish Optimization (or EFO) algorithm, inspired from the
%   object detection (electrolocation) and communication
%   (electrocommunication) bahaviors of electric fish, is the most recently
%   swarm-intelligence based algorithm proposed by Selim Yilmaz and Sevil
%   Sen.
%
%   Contact:
%   @author1 - Selim Yilmaz {selimy[at]cs.hacettepe.edu.tr}
%   @author2 - Sevil Sen    {ssen[at]cs.hacettepe.edu.tr}

clear all
clc

%   Control parameters of EFO
populationSize = 50;                        %   Population size, (M)
maxCycle =  10000;                          %   Maximum number of iterations
x = 0.989;                                  %   Balances the magnitudes of current frequency and old amplitude values
activeIndSize = 21;                         %   Subpopulation size, (K)
iterCounter = 1;                            %   Iteration counter
problemIndex = 1;                           %   Problem index, ranges from 1 to 50.
thres = 1e-5;                               %   A small constant value

%   Problem specific parameters
problem = callFunction(problemIndex);       %   A struct that gets problem information
lowerBound = problem.lowerBound;            %   Lower limit of the problem, (x_maxj)
upperBound = problem.upperBound;            %   Upper limit of the problem, (x_minj)
parameterSize = problem.parameterSize;      %   Parameter size to be optimized, (d)
name = problem.name;                        %   Problem name
fMin = problem.fMin;                        %   Global minimum of the problem


population = struct;                        %   Population struct, (N)

%   Initially all individuals are first initialized in the range of [x_maxj,
%   x_minj] then their fitness values are calculated (see Sect. 2.3.1)
for p = 1 : populationSize
    
    population(p).solution = lowerBound + (upperBound - lowerBound) * rand(1, parameterSize);
    population(p).fitness = benchmark_func(population(p).solution, problemIndex);
    population(p).isActive = true;
    
end


%   The best individual information is stored
[bestCost, index] = min( [population(:).fitness] );
bestIndividual = population(index).solution;

%   The worst individual information is also stored for frequency calculation
[worstCost, index] = max( [population(:).fitness] );
worstIndividual = population(index).solution;

globalBest = bestCost;
globalBestSol = bestIndividual;


%   Frequency (f) and amplitude (A) values are also initialized
for p = 1 : populationSize
    
    if std([population(:).fitness]) < thres,
        
        population(p).frequency = rand;
        
    else
        
        population(p).frequency = (worstCost - population(p).fitness) / (worstCost - bestCost);
        
    end
    
    population(p).amplitude = population(p).frequency;
    
end


%   Beginning of the optimization process
while (iterCounter < maxCycle && globalBest ~= fMin)
    
    for p = 1 : populationSize  %   For every individual do,
        
        newIndividual = population(p);  %   ith individual, (i)
        neighborIndividual = population;    neighborIndividual(p) = []; %   Neighbors of i, (N - {i})
        
        if (newIndividual.frequency > rand) %   Active electrolocation phase (see Sect. 2.3.2)
            
            population(p).isActive = true;  %   Broadcasts other individuals that ith individual will be in active mode
            newIndividual.solution = activeSearch(newIndividual, neighborIndividual, lowerBound, upperBound);   %   Performs search through active electrolocation
            
        else    %   Passive electrolocation phase (see Sect. 2.3.3)
            
            population(p).isActive = false; %   Broadcasts other individuals that ith individual will be in passive mode
            activePopulation = neighborIndividual( [neighborIndividual(:).isActive] );  %   Actively electrolocating individuals, (N_A)
            
            if ~isempty(activePopulation)
                
                newIndividual.solution = passiveSearch(newIndividual, activePopulation, activeIndSize); %   Performs search through passive electrolocation
                
            end
            
            if rand < rand, %   Mutates one more parameter in a stochastic manner to keep the population diverse
                
                I = randi(parameterSize);
                newIndividual.solution(I) = lowerBound + (upperBound - lowerBound) * rand;
                
            end
            
        end
        
        newIndividual.solution = boundaryCheck(newIndividual.solution, lowerBound, upperBound); %   Solution set of candidate individual, (x_cand)
        
        newIndividual.fitness = benchmark_func(newIndividual.solution, problemIndex);   %   Fitness value of candidate individual
        
        if  newIndividual.fitness < population(p).fitness   %   Accepts if better source found
            
            population(p) = newIndividual;
            
        end
        
    end
    %   The best individual information is stored
    [bestCost, index] = min( [population(:).fitness] );
    bestIndividual = population(index).solution;
    
    %   The worst individual information is also stored for frequency calculation
    [worstCost, index] = max( [population(:).fitness] );
    worstIndividual = population(index).solution;
    
    %   Frequency (f) and amplitude (A) values are updated
    for p = 1: populationSize
        
        if std([population(:).fitness]) < thres,
            
            population(p).frequency = rand;
            
        else
            
            population(p).frequency = (worstCost - population(p).fitness) / (worstCost - bestCost);
            
        end
        
        population(p).amplitude = population(p).amplitude * (x) + population(p).frequency * (1 - x);
        
    end
    
    
    if bestCost < globalBest,   %   The best cost and solution found so far are stored
        
        globalBest = bestCost;
        globalBestSol = bestIndividual;
        
    end
    
    iterCounter = iterCounter + 1;
    
    disp(['EFO  Problem:', num2str(problemIndex), '-',name,' D:',num2str(problem.parameterSize), ...
        ' iter:', num2str(iterCounter), ' Fmin:', num2str(globalBest)])
    
%   End of the iteration
end

end

function newSolution = activeSearch(newIndividual, neighborIndividual, lowerBound, upperBound)
%%  Active Electrolocation Phase

activeRange = (upperBound - lowerBound) * newIndividual.amplitude;    %   Calculates active search range

parameter = randi(size(newIndividual.solution, 2)); %   Determines jth parameter to modify

dist = getDistance(newIndividual.solution, [neighborIndividual(:).solution]);   %   Obtains cartesian distance between ith individual and its neighbors ( i.e., N \ {i} )

index = find(dist < activeRange);   %   Determines if at least a neighbor exists in the active search range

if isempty(index)   %   If no neighbor exists in the search range (see Eq. 7)
    
    newIndividual.solution(parameter) = newIndividual.solution(parameter) + (2 * rand - 1) * activeRange;
    
else    %   If at least one neighbor exists in the search range (see Eq. 6)
    
    selectedNeighbor = index(randi(size(index, 1))); %   Randomly selects one neighbor individual (k) among those inside active range area
    newIndividual.solution(parameter) = newIndividual.solution(parameter) + ...
        (neighborIndividual(selectedNeighbor).solution(parameter) - ...
        newIndividual.solution(parameter)) * (2 * rand - 1);
    
end

newSolution = newIndividual.solution;

end

function newSolution = passiveSearch(newIndividual, activePopulation, activeIndSize)
%%  Passive Electrolocation Phase

e = 1e-6;   %   A small constant used in distance calculation
D = size(newIndividual.solution, 2);    %   Parameter size of the problem

if size(activePopulation, 2) < activeIndSize,   %   If the number of active individuals is less than that of predetermined neighbor, (K)
    
    activeIndSize = size(activePopulation, 2);  %   Set K as the number of active individuals
    
end

dist = getDistance(newIndividual.solution, [activePopulation(:).solution]); %   Obtains cartesian distance between ith individual and those in active modes
p = cumsum([activePopulation(:).amplitude] ./ (dist' + e));   %   The probability values of active mode individuals (see Eq. 8)
selectedNeighbor = zeros(1, activeIndSize);

for i = 1 : activeIndSize,  %   K individuals are selected through fitness proportionate (or roulette wheel) selection (see Eq. 8)
    
    I = find (rand*p(end) < p);
    selectedNeighbor(i) = I(1);
    
end

neighbor = activePopulation(selectedNeighbor);
solutionSet = reshape([neighbor.solution], D, activeIndSize)';
xReference = sum(repmat([neighbor.amplitude]', 1, D) .* solutionSet) ./ sum(repmat([neighbor.amplitude]', 1, D));    %   A reference point is created (see Eq. 9)
newSolution = newIndividual.solution + (xReference - newIndividual.solution) .* (2 * rand(1, D) - 1);    %   Generates new location (see Eq. 10)
I = find(rand(1, D) > newIndividual.frequency); %   Determines which modified parameters will be accepted
newIndividual.solution(I) = newSolution(I);
newSolution = newIndividual.solution;

end

%   Returns the cartesian distance between individual 'newIndividualSol' and
%   its neigbors 'solutionSet' (see Eq. 5)
function dist = getDistance(newIndividualSol, solutionSet)
%%  Distance Calculator

parameterSize = size(newIndividualSol, 2);
popSize = size(solutionSet, 2) / parameterSize;
solutionSet = reshape(solutionSet, parameterSize, popSize)';
dist = sqrt(sum((solutionSet - repmat(newIndividualSol, popSize, 1)) .^2, 2));

end

%   Controls if a parameter value exceeds the space limits, if so it is
%   relocated into the bound where it exceeds
function newIndividualSol = boundaryCheck(newIndividualSol, lowerBound, upperBound)
%%  Boundary Controller

newIndividualSol = min(newIndividualSol, upperBound);
newIndividualSol = max(newIndividualSol, lowerBound);

end